defmodule Pixie.Response.Publish do
  defstruct channel: nil, client_id: nil, error: nil, advice: nil, ext: nil, id: nil
  import Pixie.Utils.Response

  @moduledoc """
  Convert an incoming `Pixie.Message.Publish` into a response.

      Response
      MUST include:  * channel
                     * successful
      MAY include:   * id
                     * error
                     * ext

  This struct contains the following keys:

    - `:channel` the channel the client is attempting to publish to.
    - `:client_id` the client ID generated by the server during handshake.
    - `:error` an error message to send to the client explaining why the
      request cannot proceed. Optional.
    - `:ext` an arbitrary map of data the server sends for use in extensions
      (usually authentication information, etc). Optional.
    - `:id` a message ID generated by the client. Optional.
    - `:advice` advice from the server about how to handle timeouts, polling
      intervals, etc. See
      [the Bayeux protocol](http://svn.cometd.org/trunk/bayeux/bayeux.html#toc_32)
      for more information.
  """

  @doc """
  Create a `Pixie.Response.Publish` struct based on some fields from the
  incoming message.
  """
  def init %Pixie.Message.Publish{}=message do
    %Pixie.Response.Publish{}
      |> put(message, :channel)
      |> put(message, :id)
  end
end
